#!/usr/bin/env python

"""
Twisted adapter.
"""

import WebStack.Twisted
from WebStack.Generic import EndOfResponse
import twisted.web.resource

class Dispatcher(twisted.web.resource.Resource):

    "A class dispatching requests to WebStack resources."

    isLeaf = 1

    def __init__(self, resource, authenticator=None, handle_errors=1):

        """
        Initialise the root application-specific 'resource' and optional
        'authenticator'. The optional 'handle_errors' parameter (if true) causes
        handlers to deal with uncaught exceptions cleanly.
        """

        twisted.web.resource.Resource.__init__(self)
        self.webstack_resource = resource
        self.webstack_authenticator = authenticator
        self.handle_errors = handle_errors

    def getChild(self, path, request):
        raise NotImplementedError, "getChild"

    def render(self, request):

        "Dispatch the given 'request' to the root application-specific resource."

        trans = WebStack.Twisted.Transaction(request)
        if self.webstack_authenticator is None or self.webstack_authenticator.authenticate(trans):
            try:
                self.webstack_resource.respond(trans)
            except EndOfResponse:
                pass
            except:
                if self.handle_errors:
                    trans.set_response_code(500) # Internal error
                else:
                    raise
        else:
            trans.set_response_code(401) # Unauthorized
            trans.set_header_value("WWW-Authenticate", '%s realm="%s"' % (
                self.webstack_authenticator.get_auth_type(), self.webstack_authenticator.get_realm()))

        trans.commit()
        request.finish()
        return twisted.web.server.NOT_DONE_YET

# vim: tabstop=4 expandtab shiftwidth=4
